//
//  Copyright (C) 2010 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#include "FdoWrapDll.h"

//Database connection pooling / management


namespace fdo
{

class Database;


class FDOWRAP_API DbPool
{
public:

    static Database* GetConnection(const wchar_t* connStr);
    static void ReleaseConnection(Database* db);
    static void CloseAllFreeConnections();
};

    
//Used to manage lifetime of connections, by releasing the connection
//it holds back to the pool when the handle goes out of scope
class FDOWRAP_API DbHandle
{
public:
    DbHandle() { _db = NULL; }
    DbHandle(Database* db);
    ~DbHandle();
    DbHandle& operator=(Database* db);

    Database* operator->() const { return _db; }
    bool operator!() const { return _db == NULL; }
    operator Database*() const { return _db; }
    
   
private:
    //We don't like those
    DbHandle(const DbHandle&);
    DbHandle& operator= (const DbHandle&);

    Database* _db;
};




    
}