//
//  Copyright (C) 2010 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#pragma once

#if !EXCLUDE_FDO

#include "BaseWrap.h"

#pragma warning(disable:4251) //dll interface warnings for FdoPtr<>, have to fix eventually

namespace fdo
{

class FDOWRAP_API TableFdo : public Table
{
public:

    TableFdo(FdoIConnection* db, const TableDef* ri);
    ~TableFdo();

    //Modification operations
    //TODO: BeginBulkInsert/EndBulkInsert
    bool Insert(const Row& row, __int64* newfid, bool skipID = false);

    bool Update(const Row& row);

    bool Delete(__int64 fid);

    //Query operations
    void Query(); //get them all
    void QueryRange(__int64 minID, __int64 maxID);
    void QueryRandomAccess() {throw;} //random access cursor
    void Query(const double bbox[4], FdoFilter* attributeFilter = NULL); //simple bbox query cursor
    const Row* Query(__int64 id); //query for specific feature
    void Query(FdoFilter*); //custom filter

    void EndQuery();

    //Aggregate operations
    void GetExtent(double bbox[4]);
    __int64 GetRowCount();
    
    //reading of query results
    const Row* Next(); //advance cursor, returns current row, or NULL if done
    const Row& At() { return *_row; } //row at current cursor position

    //TODO: make this more friendly
    Row& WR() { return *_row; } //writable row for use by inserts, updates

private:

    void InitSelect();

private:
    Row* _row;

    FdoPtr<FdoIConnection> _db;
    FdoPtr<FdoIFeatureReader> _rdr;

    FdoPtr<FdoIInsert> _insert;
    FdoPtr<FdoIDelete> _delete;
    FdoPtr<FdoIUpdate> _update;
    FdoPtr<FdoISelect> _select;

    FdoPtr<FdoInt64Value> _delid; //alias for fid value for delete command

    FdoPtr<FdoComparisonCondition> _idfilter; //filter by id
    FdoPtr<FdoInt64Value> _updid; //alias for fid value for query by id

    FdoPtr<FdoPropertyValueCollection> _pvc; //alias for the insert command's collection of prop vals

};





class FDOWRAP_API DatabaseFdo : public Database
{
public:

    DatabaseFdo();
    ~DatabaseFdo();

    static bool Create(const wchar_t* connStr/*, const DatabaseDef& schema, const CoordSysDef& def*/);
    
    //Take ownership of existing FDO connection
    bool Connect(FdoIConnection* db);

    //Initialize with a new FDO connection to the given file/db
    bool Connect(const wchar_t* connStr, bool readOnly = true);

    //Explicitly close connection
    void Disconnect();

    //Apply schema
    bool SetSchema(const DatabaseDef& def);

    //=======================
    // Non interface API

    //CreateSpatialContexts

    FdoIConnection* FdoConnection() { return FDO_SAFE_ADDREF(_db); }

private:

    bool SetCoordSys(const CoordSysDef& def);

    void InitSchema();
    
    FdoIConnection* _db;
};




}



#endif