//
//  Copyright (C) 2010 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#pragma once

#if !EXCLUDE_OGR

#include "BaseWrap.h"

class OGRLayer;
class OGRDataSource;
class OGRFeature;
class OGRGeometry;

namespace fdo
{

class TableOgr : public Table
{
public:

    TableOgr(OGRLayer* layer, const TableDef* td);
    ~TableOgr();

    //Modification operations
    //TODO: BeginBulkInsert/EndBulkInsert
    bool Insert(const Row& row, __int64* newfid, bool skipID = false);

    bool Update(const Row& row);

    bool Delete(__int64 fid);

    //Query operations
    void Query(); //get them all
    void QueryRange(__int64 minID, __int64 maxID);
    void QueryRandomAccess(); //random access cursor
    void Query(const double bbox[4], FdoFilter* attributeFilter = NULL); //simple bbox query cursor
    const Row* Query(__int64 id); //query for specific feature
    void Query(FdoFilter*); //custom filter

    void EndQuery();

    //Aggregate operations
    void GetExtent(double bbox[4]);
    __int64 GetRowCount();
    
    //reading of query results
    const Row* Next(); //advance cursor, returns current row, or NULL if done
    const Row& At() { return *_row; } //row at current cursor position

    //TODO: make this more friendly
    Row& WR() { return *_row; } //writable row for use by inserts, updates

private:

    void InitSelect();

    void FeatureToRow(Row& r, OGRFeature* f);
    void RowToFeature(OGRFeature* f, const Row& r);
    void GeomToValue(Value& v, OGRGeometry* geom);
    OGRGeometry* ValueToGeom(const Value& v);

    Row* _row;

    unsigned char* _fgfbuf;
    int _fgflen;
    unsigned char* _wkbbuf;
    int _wkblen;

    OGRLayer* _layer;
};



class DatabaseOgr : public Database
{
public:

    DatabaseOgr();
    ~DatabaseOgr();

    //static bool Create(const wchar_t* connStr/*, const DatabaseDef& schema, const CoordSysDef& def*/);
    
    //Initialize with a new FDO connection to the given file/db
    bool Connect(const wchar_t* connStr, bool readOnly);

    //Explicitly close connection
    void Disconnect();

    //Apply schema
    bool SetSchema(const DatabaseDef& def);

private:

    void InitSchema();
    

    OGRDataSource* _db;
        
};

}


#endif