//
//  Copyright (C) 2010 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#include "stdafx.h"
#include "Value.h"


namespace fdo {


//=============================================================================
//=============================================================================
// Value
//=============================================================================
//=============================================================================

void Value::FreeMem()
{
    int t = Type();

    if (t == vtString)
    {
        free(pstr);
        pstr = NULL;
        size = 0;
    }
    else if (t == vtBlob && size > 0)
    {
        free(pblob);
        pblob = NULL;
        size = 0;
    }
    else if (t != vtBlobNotOwn
             && ((intptr_t)ptostr & ~0xf) != NULL)
    {
        //free the cached string representation for simple types
        _aligned_free((void*)((intptr_t)ptostr & ~0xf));
        ptostr = (wchar_t*)((intptr_t)ptostr & 0xf);
    }
}

void Value::SetString(const wchar_t* str, bool own)
{
    if (!str)
    {
        size = 0;
        type = vtNull;
        ival = 0;
    }
    else if (own)
    {
        size = 0; //strings are null terminanted, no need to store size
        type = vtString;
        pstr = (wchar_t*)malloc(sizeof(wchar_t)*(wcslen(str)+1));
        wcscpy(pstr, str);
    }
    else
    {
        size = 0; //strings are null terminanted, no need to store size
        type = vtStringNotOwn;
        pstr = (wchar_t*)str;
    }
}

void Value::SetBlob(const void* blob, int len, bool own)
{
    if (!blob)
    {
        size = 0;
        type = vtNull;
        ival = 0;
    }
    else if (own)
    {
        size = len;
        type = vtBlob;
        pblob = (unsigned char*)malloc(size);
        memcpy(pblob, blob, size);
    }
    else
    {
        size = len;
        type = vtBlobNotOwn;
        pblob = (void*)blob;
    }
   
}

Value::~Value()
{
    FreeMem();
}

const wchar_t* Value::CachedStringPtr() const
{
    wchar_t* ptr = (wchar_t*)((intptr_t)ptostr & ~0xf); //slice out the last 4 bits
    if (!ptr)
    {
        wchar_t tmp[64];
        int t = Type();

        switch (t)
        {
        case vtInteger: swprintf(tmp, 64, L"%I64d", ival); break;
        case vtReal: swprintf(tmp, 64, L"%.16g", rval); break;//TODO: check the printf format here
        case vtBool: swprintf(tmp, 64, L"%s", bval ? "true" : "false"); break;
        default: *tmp = 0; break;
        }

        ptr = (wchar_t*)_aligned_malloc(sizeof(wchar_t)*(wcslen(tmp)+1), 16);
        wcscpy(ptr, tmp);

        ptostr = (wchar_t*)((intptr_t)ptr | t);
    }
    return ptr;
}



}