//
//  Copyright (C) 2010 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#pragma once

//=============================================================
//  String conversion utils

#include "utf8_.h"
#include <assert.h>


static std::string W2A_SLOW(const wchar_t* input)
{
    assert( input!=NULL );
    size_t wlen = wcslen(input);
    int mbslen = (int) wlen * 4 + 1;
    char* mbs = (char*)alloca(mbslen);
    //WideCharToMultiByte(CP_UTF8, 0, input, -1, mbs, mbslen, 0, 0);
    ut_utf8_from_unicode(input, wlen, mbs, mbslen);
    return std::string(mbs);
}

static std::wstring A2W_SLOW(const char* input)
{
    assert( input!=NULL );
    int wlen = (int)strlen(input) + 1;
    wchar_t* ws = (wchar_t*)alloca(sizeof(wchar_t)*wlen);
    //MultiByteToWideChar(CP_UTF8, 0, input, -1, ws, wlen);
    ut_utf8_to_unicode(input, wlen, ws, wlen);
    return std::wstring(ws);
}

static int A2W_FAST(wchar_t* dst, int wlen, const char* src, int clen)
{
    return ut_utf8_to_unicode(src, clen, dst, wlen);
}

static int W2A_FAST(char* dst, int clen, const wchar_t* src, int wlen)
{
    return ut_utf8_from_unicode(src, wlen, dst, clen);
}

static void ALX_ERROR(const wchar_t* msg)
{
    fprintf(stderr, "Error: %ls\n", msg);
}
static void ALX_ERROR(const char* msg)
{
    fprintf(stderr, "Error: %s\n", msg);
}




static bool DateFromString(FdoDateTime& dt, const wchar_t* val)
{
    // to avoid stack corruption declare following variables
    // since swscanf expects "Type of argument" = "Pointer to int"
    int year = -1;
    int month = -1;
    int day = -1;
    int hour = -1;
    int minute = -1;
    float seconds = 0.0f;

    const wchar_t* valtmp = val;
    int formatDate = 0x00;
    while(*valtmp != '\0' && valtmp-val < 30)
    {
        if (*valtmp == '-')
            formatDate |= 0x01;
        else if (*valtmp == ':')
        {
            formatDate |= 0x02;
            break;
        }
        else if (*valtmp == ' ')
        {
            formatDate |= 0x04;
            break;
        }
        else if (*valtmp == 'T')
        {
            formatDate |= 0x08;
            break;
        }
        valtmp++;
    }
    
    int res = 0;
    if (formatDate != 0x00)
    {
        if ((formatDate&(~0x01)) == 0x00) // it's a date
            res = swscanf(val, L"%d-%d-%d", &year, &month, &day);
        else if ((formatDate&(~0x02)) == 0x00) // it's a time
            res = swscanf(val, L"%d:%d:%f", &hour, &minute, &seconds);
        else if ((formatDate&(~0x07)) == 0x00) // it's a datetime using space
            res = swscanf(val, L"%d-%d-%d %d:%d:%f", &year, &month, &day, &hour, &minute, &seconds);
        else  if ((formatDate&(~0x0B)) == 0x00) // it's a datetime using T
            res = swscanf(val, L"%d-%d-%dT%d:%d:%f", &year, &month, &day, &hour, &minute, &seconds);
    }
    if (res != 3 && res != 5 && res != 6)
    {
        return false;
    }
    
    dt.year = year;
    dt.month = month;
    dt.day = day;
    dt.hour = hour;
    dt.minute = minute;
    dt.seconds = seconds;
    return true;
}

static bool DateToString(FdoDateTime& dt, wchar_t* s, int nBytes)
{
    if (dt.IsDateTime())
    {
        _snwprintf(s, nBytes, L"%04d-%02d-%02dT%02d:%02d:%06.3f", dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.seconds);
    }
    else if (dt.IsDate())
    {
        _snwprintf(s, nBytes, L"%04d-%02d-%02d", dt.year, dt.month, dt.day);
    }
    else if (dt.IsTime())
    {
        _snwprintf(s, nBytes, L"%02d:%02d:%06.3f", dt.hour, dt.minute, dt.seconds);
    }
    else
        return false;

    return true;
}
